@@ -5,7 +5,7 @@ from importlib import metadata as _metadata
 __all__ = ["__version__"]
 
 
-def __getattr__(name: str):
+def __getattr__(name: str) -> str:
     """Return dynamic module attributes supported by the package.
 
     Args:
@@ -22,7 +22,7 @@ def __getattr__(name: str):
     raise AttributeError(f"module {__name__!r} has no attribute {name!r}")
 
 
-def __dir__():
+def __dir__() -> list[str]:
     """Return the list of attributes available on the package module.
 
     Returns:
