@@ -2,8 +2,9 @@
 
 This module exposes a thin wrapper around DSPy programs so the rest of the
 codebase can request classifications without depending directly on DSPy.
-When DSPy is unavailable, the engine falls back to lightweight heuristics so
-we can still exercise the higher-level pipeline in development and tests.
+When DSPy is unavailable, the engine falls back to lightweight heuristics only
+when ``DORGY_USE_FALLBACKS=1`` so we can still exercise the higher-level
+pipeline in development and tests.
 """
 
 from __future__ import annotations
@@ -24,7 +25,9 @@ except ImportError:  # pragma: no cover - executed when DSPy absent
     dspy = None
 
 from dorgy.classification.dspy_logging import configure_dspy_logging
+from dorgy.classification.exceptions import LLMUnavailableError
 from dorgy.config.models import LLMSettings
+from dorgy.shutdown import ShutdownRequested, check_for_shutdown
 
 from .models import (
     ClassificationBatch,
@@ -42,29 +45,78 @@ _CATEGORY_ALIASES = {
     "image": "Media/Images",
 }
 
+_CONFIDENCE_PATTERN = re.compile(r"-?\d+(?:\.\d+)?")
+
+
+def _coerce_confidence(raw: object, *, default: float = 0.5) -> float:
+    """Return a normalised confidence score between 0 and 1.
+
+    Args:
+        raw: Value returned by the language model for confidence.
+        default: Fallback value to use when the model omits a numeric score.
+
+    Returns:
+        float: Confidence bounded to the inclusive range ``[0.0, 1.0]``.
+    """
+
+    numeric: Optional[float] = None
+
+    if isinstance(raw, (int, float)):
+        numeric = float(raw)
+    elif isinstance(raw, str):
+        stripped = raw.strip()
+        if stripped:
+            candidate = stripped.replace(",", ".")
+            match = _CONFIDENCE_PATTERN.search(candidate)
+            if match:
+                try:
+                    numeric = float(match.group())
+                except ValueError:
+                    numeric = None
+            if numeric is None:
+                lowered = stripped.lower()
+                if "high" in lowered:
+                    numeric = 0.9
+                elif "medium" in lowered:
+                    numeric = 0.6
+                elif "low" in lowered:
+                    numeric = 0.2
+    if numeric is None:
+        numeric = default
+    if not (numeric >= 0.0):
+        numeric = default
+    return max(0.0, min(1.0, numeric))
+
 
 class ClassificationEngine:
     """Apply DSPy programs to classify and rename files.
 
-    The engine prefers DSPy when available, but automatically falls back to
-    heuristic classification so development and tests can proceed without the
-    optional dependency.
+    The engine prefers DSPy when available, but only falls back to heuristic
+    classification when ``DORGY_USE_FALLBACKS=1`` so development and tests can
+    proceed without the optional dependency.
     """
 
     def __init__(self, settings: Optional[LLMSettings] = None) -> None:
-        use_fallback = os.getenv("DORGY_USE_FALLBACK") == "1"
+        legacy_flag = os.getenv("DORGY_USE_FALLBACK")
+        if legacy_flag is not None:
+            LOGGER.warning(
+                "DORGY_USE_FALLBACK is deprecated; set DORGY_USE_FALLBACKS=1 to enable heuristics."
+            )
+
+        use_fallback = os.getenv("DORGY_USE_FALLBACKS") == "1"
         self._settings = settings or LLMSettings()
+        self._use_fallbacks = use_fallback
 
         if use_fallback:
             self._has_dspy = False
             self._program = None
-            LOGGER.info("Heuristic fallback enabled by DORGY_USE_FALLBACK=1.")
+            LOGGER.info("Heuristic fallback enabled by DORGY_USE_FALLBACKS=1.")
             return
 
         if dspy is None:
-            raise RuntimeError(
+            raise LLMUnavailableError(
                 "DSPy is not installed. Install the `dspy` package (and any provider-specific "
-                "dependencies), or set DORGY_USE_FALLBACK=1 to enable the heuristic classifier."
+                "dependencies), or set DORGY_USE_FALLBACKS=1 to enable the heuristic classifier."
             )
 
         configure_dspy_logging()
@@ -105,6 +157,7 @@ class ClassificationEngine:
             ClassificationBatch: Aggregated decisions and errors.
         """
         requests = list(requests)
+        check_for_shutdown()
         batch = ClassificationBatch(decisions=[None] * len(requests), errors=[])
         if not requests:
             return batch
@@ -148,22 +201,12 @@ class ClassificationEngine:
         def _classify_single(
             index: int, request: ClassificationRequest
         ) -> tuple[int, Optional[ClassificationDecision], Optional[str]]:
+            check_for_shutdown()
             start = time.perf_counter()
             worker_id = _worker_id()
             _notify_progress(index, request, worker_id, "start", None, None)
             try:
-                if self._has_dspy and self._program is not None:
-                    decision = self._classify_with_dspy(request)
-                else:
-                    decision = self._fallback_classify(request)
-                duration = time.perf_counter() - start
-                LOGGER.debug(
-                    "Classification completed for %s in %.2fs",
-                    request.descriptor.path,
-                    duration,
-                )
-                _notify_progress(index, request, worker_id, "complete", duration, None)
-                return index, decision, None
+                decision = self._classify_request(request)
             except Exception as exc:  # pragma: no cover - defensive safeguard
                 duration = time.perf_counter() - start
                 LOGGER.warning(
@@ -173,10 +216,25 @@ class ClassificationEngine:
                     exc,
                 )
                 _notify_progress(index, request, worker_id, "complete", duration, exc)
-                return index, None, f"{request.descriptor.path}: {exc}"
+                if not self._use_fallbacks or isinstance(exc, LLMUnavailableError):
+                    raise
+                LOGGER.info(
+                    "Falling back to heuristic classification for %s after DSPy failure.",
+                    request.descriptor.path,
+                )
+                decision = self._fallback_classify(request)
+            duration = time.perf_counter() - start
+            LOGGER.debug(
+                "Classification completed for %s in %.2fs",
+                request.descriptor.path,
+                duration,
+            )
+            _notify_progress(index, request, worker_id, "complete", duration, None)
+            return index, decision, None
 
         if worker_count == 1:
             for idx, request in enumerate(requests):
+                check_for_shutdown()
                 index, decision, error = _classify_single(idx, request)
                 batch.decisions[index] = decision
                 if error:
@@ -188,11 +246,17 @@ class ClassificationEngine:
                 executor.submit(_classify_single, idx, request): idx
                 for idx, request in enumerate(requests)
             }
-            for future in as_completed(future_map):
-                index, decision, error = future.result()
-                batch.decisions[index] = decision
-                if error:
-                    batch.errors.append(error)
+            try:
+                for future in as_completed(future_map):
+                    check_for_shutdown()
+                    index, decision, error = future.result()
+                    batch.decisions[index] = decision
+                    if error:
+                        batch.errors.append(error)
+            except ShutdownRequested:
+                for future in future_map:
+                    future.cancel()
+                raise
 
         return batch
 
@@ -258,6 +322,20 @@ class ClassificationEngine:
     # Internal helpers                                                   #
     # ------------------------------------------------------------------ #
 
+    def _classify_request(self, request: ClassificationRequest) -> ClassificationDecision:
+        """Return a classification decision using DSPy or heuristics."""
+
+        if self._has_dspy and self._program is not None:
+            return self._classify_with_dspy(request)
+
+        if self._use_fallbacks:
+            return self._fallback_classify(request)
+
+        raise LLMUnavailableError(
+            "Heuristic fallbacks are disabled. Set DORGY_USE_FALLBACKS=1 to allow heuristics when "
+            "the configured language model cannot be used."
+        )
+
     def _configure_language_model(self) -> None:
         """Configure the DSPy language model according to LLM settings."""
         default_settings = LLMSettings()
@@ -265,33 +343,14 @@ class ClassificationEngine:
             [
                 self._settings.api_base_url,
                 self._settings.api_key,
-                self._settings.provider != default_settings.provider,
                 self._settings.model != default_settings.model,
             ]
         )
         if not configured:
-            raise RuntimeError(
+            raise LLMUnavailableError(
                 "LLM configuration is incomplete. Update ~/.dorgy/config.yaml with valid values "
-                "for the llm block (provider/model/api_key or api_base_url), or set "
-                "DORGY_USE_FALLBACK=1 to force the heuristic classifier."
-            )
-
-        api_key_missing = self._settings.api_key is None
-
-        if self._settings.api_base_url and api_key_missing:
-            # Local gateways (e.g., Ollama) don't require authentication, so supply an empty string.
-            self._settings.api_key = ""
-            api_key_missing = False
-
-        if (
-            self._settings.provider
-            and self._settings.provider != "local"
-            and self._settings.api_base_url is None
-            and api_key_missing
-        ):
-            raise RuntimeError(
-                "llm.provider is set to a remote provider but llm.api_key is missing. Provide the "
-                "API key or set DORGY_USE_FALLBACK=1 to use the heuristic classifier."
+                "for the llm block (model/api_key/api_base_url), or set "
+                "DORGY_USE_FALLBACKS=1 to force the heuristic classifier."
             )
 
         lm_kwargs: dict[str, object] = {
@@ -302,22 +361,38 @@ class ClassificationEngine:
 
         if self._settings.api_base_url:
             lm_kwargs["api_base"] = self._settings.api_base_url
-        elif self._settings.provider:
-            lm_kwargs["provider"] = self._settings.provider
-
-        if self._settings.api_key is not None:
+        if self._settings.api_key is not None and self._settings.api_key != "":
             lm_kwargs["api_key"] = self._settings.api_key
 
         try:
             language_model = dspy.LM(**lm_kwargs)
             dspy.settings.configure(lm=language_model)
         except Exception as exc:  # pragma: no cover - DSPy misconfiguration
-            raise RuntimeError(
+            raise LLMUnavailableError(
                 "Unable to configure the DSPy language model. Verify your llm.* settings "
-                "(provider/model/api_key/api_base_url) or set DORGY_USE_FALLBACK=1 to use the "
+                "(model/api_key/api_base_url) or set DORGY_USE_FALLBACKS=1 to use the "
                 "heuristic classifier."
             ) from exc
 
+    def _fallback_confidence(self, classification: object) -> float:
+        """Return a conservative confidence when the model omits a score.
+
+        Args:
+            classification: Raw DSPy classification object.
+
+        Returns:
+            float: Calibrated confidence derived from category and tag signals.
+        """
+
+        primary = getattr(classification, "primary_category", None)
+        if isinstance(primary, str) and primary.strip():
+            if primary.strip().lower() != "general":
+                return 0.6
+        tags = getattr(classification, "tags", None) or []
+        if tags:
+            return 0.5
+        return 0.4
+
     def _classify_with_dspy(self, request: ClassificationRequest) -> ClassificationDecision:
         """Leverage DSPy program to classify the file.
 
@@ -328,7 +403,7 @@ class ClassificationEngine:
             ClassificationDecision: Result derived from DSPy output.
         """
         if self._program is None:
-            raise RuntimeError("DSPy program has not been initialised")
+            raise LLMUnavailableError("DSPy program has not been initialised")
 
         descriptor = request.descriptor
         metadata_dump = json.dumps(descriptor.metadata, ensure_ascii=False)
@@ -341,12 +416,21 @@ class ClassificationEngine:
             "prompt": request.prompt or "",
         }
 
+        confidence_guidance = (
+            "When providing the confidence value, respond with a decimal between 0 and 1 "
+            "(for example 0.73). Avoid percentages or verbal qualifiers."
+        )
+        if payload["prompt"]:
+            payload["prompt"] = f"{payload['prompt']}\n\n{confidence_guidance}"
+        else:
+            payload["prompt"] = confidence_guidance
+
         classification, rename = self._program(payload)
 
-        try:
-            confidence = float(classification.confidence)
-        except (ValueError, TypeError):
-            confidence = 0.0
+        confidence_raw = getattr(classification, "confidence", None)
+        confidence = _coerce_confidence(confidence_raw)
+        if confidence <= 0.0:
+            confidence = self._fallback_confidence(classification)
 
         secondary = classification.secondary_categories or []
         tags = classification.tags or []
