@@ -2,6 +2,7 @@
 
 from __future__ import annotations
 
+import logging
 import os
 import queue
 import shutil
@@ -31,14 +32,12 @@ from dorgy.classification import (
     VisionCache,
     VisionCaptioner,
 )
-from dorgy.cli_support import (
+from dorgy.cli.helpers.classification import run_classification, zip_decisions
+from dorgy.cli.helpers.organization import collect_error_payload, compute_org_counts
+from dorgy.cli.helpers.state import (
     build_original_snapshot,
-    collect_error_payload,
-    compute_org_counts,
     descriptor_to_record,
     relative_to_collection,
-    run_classification,
-    zip_decisions,
 )
 from dorgy.config import DorgyConfig
 from dorgy.ingestion import IngestionPipeline
@@ -49,14 +48,28 @@ from dorgy.ingestion.models import IngestionResult
 from dorgy.organization.executor import OperationExecutor
 from dorgy.organization.models import DeleteOperation, OperationPlan
 from dorgy.organization.planner import OrganizerPlanner
+from dorgy.search import (
+    SearchEntry,
+    SearchIndex,
+    SearchIndexError,
+    delete_entries,
+    descriptor_document_text,
+    ensure_index,
+    update_entries,
+)
+from dorgy.shutdown import ShutdownRequested, check_for_shutdown, shutdown_requested
 from dorgy.state import (
     CollectionState,
+    FileRecord,
     MissingStateError,
     OperationEvent,
+    SearchState,
     StateError,
     StateRepository,
 )
 
+LOGGER = logging.getLogger(__name__)
+
 
 @dataclass(slots=True)
 class WatchBatchResult:
@@ -147,29 +160,40 @@ class WatchService:
         config: DorgyConfig,
         *,
         roots: Iterable[Path],
-        prompt: Optional[str],
+        classification_prompt: Optional[str],
+        structure_prompt: Optional[str],
         output: Optional[Path],
         dry_run: bool,
         recursive: bool,
         debounce_override: Optional[float] = None,
         allow_deletions: bool = False,
+        with_search: bool = False,
+        without_search: bool = False,
+        embedding_function: Any | None = None,
     ) -> None:
         """Initialize the watch service.
 
         Args:
             config: Loaded Dorgy configuration.
             roots: Iterable of directory roots to monitor.
-            prompt: Optional classification prompt override.
+            classification_prompt: Optional classification prompt override.
+            structure_prompt: Optional structure planning prompt override.
             output: Optional destination root when operating in copy-mode.
             dry_run: Whether to avoid filesystem mutations.
             recursive: Whether to monitor subdirectories.
             debounce_override: Optional debounce interval override in seconds.
             allow_deletions: Whether to remove records when files are deleted or
                 moved outside monitored roots.
+            with_search: Force-enable search indexing for batches.
+            without_search: Force-disable search indexing for batches.
+            embedding_function: Optional Chromadb embedding function override.
         """
 
+        if with_search and without_search:
+            raise ValueError("--with-search cannot be combined with --without-search.")
         self._config = config
-        self._prompt = prompt
+        self._classification_prompt = classification_prompt
+        self._structure_prompt = structure_prompt or classification_prompt
         self._roots = [root.expanduser().resolve() for root in roots]
         self._output = output.expanduser().resolve() if output else None
         self._dry_run = dry_run
@@ -201,6 +225,11 @@ class WatchService:
         self._backoff: dict[Path, float] = defaultdict(lambda: self._initial_backoff)
         self._copy_mode = False
         self._allow_deletions = allow_deletions
+        self._search_override: bool | None = (
+            True if with_search else False if without_search else None
+        )
+        self._search_embedding_function = embedding_function
+        self._search_indices: dict[Path, SearchIndex] = {}
         if self._output is not None:
             if len(self._roots) != 1:
                 raise ValueError("--output requires a single source root.")
@@ -217,8 +246,10 @@ class WatchService:
             list[WatchBatchResult]: Results produced for roots that yielded work.
         """
 
+        check_for_shutdown()
         results: list[WatchBatchResult] = []
         for root in self._roots:
+            check_for_shutdown()
             batch = self._run_batch(root, [WatchEvent(kind="scan", src=root)])
             if batch is not None:
                 results.append(batch)
@@ -281,6 +312,9 @@ class WatchService:
         flush_deadline: Optional[float] = None
 
         while not self._stop_event.is_set():
+            if shutdown_requested():
+                self.stop()
+                break
             timeout: Optional[float] = None
             if flush_deadline is not None:
                 timeout = max(0.0, flush_deadline - time.monotonic())
@@ -331,12 +365,17 @@ class WatchService:
             pending: Mapping of roots to pending watch events awaiting processing.
             callback: Callable used to surface completed batch results.
         """
+        check_for_shutdown()
         for root, events in list(pending.items()):
             if not events:
                 continue
             triggered = list(events)
             try:
+                check_for_shutdown()
                 batch = self._run_batch(root, triggered)
+            except ShutdownRequested:
+                self.stop()
+                raise
             except Exception as exc:  # pragma: no cover - defensive branch
                 self._log_failure(root, [event.src for event in triggered], exc)
                 backoff = self._backoff[root]
@@ -361,6 +400,7 @@ class WatchService:
             Optional[WatchBatchResult]: Populated batch result, or ``None`` when no
             work was required.
         """
+        check_for_shutdown()
         event_list = list(watch_events)
         if not event_list:
             return None
@@ -449,6 +489,9 @@ class WatchService:
         plan = OperationPlan()
         final_path_map: dict[Path, Path] = {}
         file_entries: list[dict[str, Any]] = []
+        search_entries: list[SearchEntry] = []
+        search_notes: list[str] = []
+        vision_warning: str | None = None
 
         if ingestion_inputs:
             cache = self._classification_caches.get(source_root)
@@ -468,7 +511,9 @@ class WatchService:
                 try:
                     vision_captioner = VisionCaptioner(self._config.llm, cache=vision_cache)
                 except RuntimeError as exc:
-                    raise RuntimeError(str(exc)) from exc
+                    vision_warning = f"Vision captioning disabled: {exc}"
+                    LOGGER.warning("%s", vision_warning)
+                    vision_captioner = None
 
             max_size_bytes = None
             if self._config.processing.max_file_size_mb > 0:
@@ -485,24 +530,28 @@ class WatchService:
                 scanner=scanner,
                 detector=TypeDetector(),
                 hasher=HashComputer(),
-                extractor=MetadataExtractor(),
+                extractor=MetadataExtractor(
+                    preview_char_limit=self._config.processing.preview_char_limit
+                ),
                 processing=self._config.processing,
                 staging_dir=staging_dir,
                 allow_writes=not self._dry_run,
                 vision_captioner=vision_captioner,
             )
 
-            result = pipeline.run(ingestion_inputs, prompt=self._prompt)
+            check_for_shutdown()
+            result = pipeline.run(ingestion_inputs, prompt=self._classification_prompt)
             if not self._dry_run and vision_captioner is not None:
                 vision_captioner.save_cache()
             parallel_workers = max(1, self._config.processing.parallel_workers)
+            check_for_shutdown()
             classification_batch = run_classification(
                 result.processed,
-                self._prompt,
-                source_root,
-                self._dry_run,
-                self._config,
-                cache,
+                classification_prompt=self._classification_prompt,
+                root=source_root,
+                dry_run=self._dry_run,
+                config=self._config,
+                cache=cache,
                 on_progress=None,
                 max_workers=parallel_workers,
             )
@@ -510,6 +559,7 @@ class WatchService:
             paired = list(zip_decisions(classification_batch, result.processed))
             confidence_threshold = self._config.ambiguity.confidence_threshold
             for decision, descriptor in paired:
+                check_for_shutdown()
                 if decision is not None and decision.confidence < confidence_threshold:
                     decision.needs_review = True
                     if descriptor.path not in result.needs_review:
@@ -527,6 +577,7 @@ class WatchService:
             move_map = {operation.source: operation.destination for operation in plan.moves}
 
             for decision, descriptor in paired:
+                check_for_shutdown()
                 original_path = descriptor.path
                 rename_target = rename_map.get(original_path)
                 move_key = rename_target if rename_target is not None else original_path
@@ -567,6 +618,9 @@ class WatchService:
                 conflict_strategy=self._config.organization.conflict_resolution,
             )
 
+        if vision_warning:
+            plan.notes.append(vision_warning)
+
         delete_operations = [
             DeleteOperation(
                 path=request.path,
@@ -585,6 +639,8 @@ class WatchService:
         triggered_paths = sorted(triggered_paths_set)
         batch_id = self._next_batch_id()
         notes = list(plan.notes)
+        if search_notes:
+            notes.extend(search_notes)
 
         executed_requests: list[RemovalRequest] = []
         suppressed_requests: list[tuple[RemovalRequest, str]] = []
@@ -664,6 +720,12 @@ class WatchService:
                 )
                 notes.append(dryrun_message)
 
+        llm_metadata = self._config.llm.runtime_metadata()
+        fallbacks_enabled = os.getenv("DORGY_USE_FALLBACKS") == "1"
+        llm_metadata["fallbacks_enabled"] = fallbacks_enabled
+        fallback_text = "enabled" if fallbacks_enabled else "disabled"
+        llm_metadata["summary"] = f"{self._config.llm.runtime_summary()}, fallbacks={fallback_text}"
+
         json_payload: dict[str, Any] = {
             "context": {
                 "batch_id": batch_id,
@@ -671,7 +733,10 @@ class WatchService:
                 "destination_root": target_root.as_posix(),
                 "copy_mode": self._copy_mode,
                 "dry_run": self._dry_run,
-                "prompt": self._prompt,
+                "classification_prompt": self._classification_prompt,
+                "structure_prompt": self._structure_prompt,
+                # Backwards compatibility: retain legacy key.
+                "prompt": self._classification_prompt,
                 "allow_deletions": self._allow_deletions,
                 "triggered_paths": [path.as_posix() for path in triggered_paths],
                 "started_at": batch_started_at.isoformat(),
@@ -684,6 +749,7 @@ class WatchService:
             "removals": removals_payload,
             "suppressed_deletions": suppressed_payload,
         }
+        json_payload["context"]["llm"] = llm_metadata
 
         if self._dry_run:
             batch_completed_at = datetime.now(timezone.utc)
@@ -730,6 +796,30 @@ class WatchService:
             state = self._repository.load(target_root)
         except MissingStateError:
             state = CollectionState(root=str(target_root))
+        if state.search is None:
+            state.search = SearchState()
+        search_index: SearchIndex | None = None
+        search_enabled = self._should_enable_search(state)
+        state.search.enabled = search_enabled
+        if not search_enabled:
+            state.search.last_indexed_at = None
+        if search_enabled:
+            try:
+                search_index = self._search_indices.get(target_root)
+                if search_index is None:
+                    search_index = ensure_index(
+                        target_root,
+                        state,
+                        embedding_function=self._search_embedding_function,
+                    )
+                    self._search_indices[target_root] = search_index
+                else:
+                    search_index.initialize()
+                    state.search.enabled = True
+            except SearchIndexError as exc:
+                search_index = None
+                state.search.enabled = False
+                search_notes.append(f"Search indexing skipped: {exc}")
 
         snapshot = build_original_snapshot([descriptor for _, descriptor in paired], source_root)
         try:
@@ -772,14 +862,50 @@ class WatchService:
 
             record = descriptor_to_record(descriptor, decision, target_root)
 
-            state.files.pop(old_relative, None)
+            previous_record = state.files.pop(old_relative, None)
+            if previous_record is not None:
+                record.document_id = previous_record.document_id
             state.files[record.path] = record
 
+        if search_index is not None:
+            for _, descriptor in paired:
+                document_text = descriptor_document_text(descriptor)
+                if not document_text:
+                    continue
+                relative = relative_to_collection(descriptor.path, target_root)
+                existing_record = state.files.get(relative)
+                if existing_record is None:
+                    continue
+                entry = SearchEntry.from_record(
+                    existing_record,
+                    document_text,
+                    extra_metadata={"mime_type": descriptor.mime_type, "source": "watch"},
+                )
+                search_entries.append(entry)
+            if search_entries:
+                try:
+                    update_entries(search_index, state, search_entries)
+                except SearchIndexError as exc:
+                    search_notes.append(f"Search indexing skipped: {exc}")
+                    state.search.enabled = False
+
         removed_relatives: list[str] = []
+        removed_doc_ids: list[str] = []
         for request in executed_requests:
             relative = relative_to_collection(request.path, target_root)
-            if state.files.pop(relative, None) is not None:
+            removed_record: FileRecord | None = None
+            if relative in state.files:
+                removed_record = state.files.pop(relative)
+            if removed_record is not None:
                 removed_relatives.append(relative)
+                removed_doc_ids.append(removed_record.document_id)
+
+        if search_index is not None and removed_doc_ids:
+            try:
+                delete_entries(search_index, state, list(dict.fromkeys(removed_doc_ids)))
+            except SearchIndexError as exc:
+                search_notes.append(f"Search indexing skipped: {exc}")
+                state.search.enabled = False
 
         delete_events: list[OperationEvent] = []
         for request in executed_requests:
@@ -906,6 +1032,17 @@ class WatchService:
         except OSError:
             pass
 
+    def _should_enable_search(self, state: CollectionState) -> bool:
+        """Return whether search indexing should run for the batch."""
+
+        if self._search_override is True:
+            return True
+        if self._search_override is False:
+            return False
+        if state.search.enabled:
+            return True
+        return self._config.search.auto_enable_watch
+
     def _should_ignore_path(self, path: Path) -> bool:
         """Return whether ``path`` should be ignored for watch processing."""
 
