@@ -26,6 +26,7 @@ def _env_with_home(tmp_path: Path) -> dict[str, str]:
     """
     env = dict(os.environ)
     env["HOME"] = str((tmp_path / "home"))
+    env.setdefault("DORGY_USE_FALLBACKS", "1")
     return env
 
 
@@ -53,7 +54,7 @@ def test_cli_org_persists_state(tmp_path: Path) -> None:
     assert record.get("hash")
     assert "Documents" in record.get("categories", [])
     assert record.get("rename_suggestion") == "doc"
-    assert record.get("needs_review") is True
+    assert record.get("needs_review") is False
     final_path = root / "documents" / "doc.txt"
     assert final_path.exists()
     snapshot_path = root / ".dorgy" / "orig.json"
@@ -85,7 +86,7 @@ def test_cli_org_classification_updates_state(tmp_path: Path) -> None:
     assert "Documents" in record["categories"]
     assert record["rename_suggestion"] == "invoice"
     assert record.get("confidence") is not None
-    assert record.get("needs_review") is True
+    assert record.get("needs_review") is False
 
 
 def test_cli_org_dry_run(tmp_path: Path) -> None:
@@ -104,6 +105,7 @@ def test_cli_org_dry_run(tmp_path: Path) -> None:
     result = runner.invoke(cli, ["org", str(root), "--dry-run"], env=env)
 
     assert result.exit_code == 0
+    assert "LLM configuration:" in result.output
     assert "Dry run" in result.output
     assert not (root / ".dorgy").exists()
 
@@ -160,6 +162,9 @@ def test_cli_org_renames_files_when_enabled(tmp_path: Path) -> None:
 
     runner = CliRunner()
     env = _env_with_home(tmp_path)
+    config_path = Path(env["HOME"]) / ".dorgy" / "config.yaml"
+    config_path.parent.mkdir(parents=True, exist_ok=True)
+    config_path.write_text("organization:\n  rename_files: true\n", encoding="utf-8")
 
     result = runner.invoke(cli, ["org", str(root)], env=env)
 
@@ -217,7 +222,14 @@ def test_cli_org_records_vision_metadata(tmp_path: Path, monkeypatch: pytest.Mon
     runner = CliRunner()
     result = runner.invoke(
         cli,
-        ["org", str(root), "--prompt", "Highlight receipts"],
+        [
+            "org",
+            str(root),
+            "--classify-prompt",
+            "Highlight receipts",
+            "--structure-prompt",
+            "Group receipts by vendor",
+        ],
         env=env,
     )
 
@@ -239,7 +251,7 @@ def test_cli_org_prompt_file_overrides_inline_prompt(
     tmp_path: Path,
     monkeypatch: pytest.MonkeyPatch,
 ) -> None:
-    """Ensure `--prompt-file` overrides inline prompts for organization runs."""
+    """Ensure `--classify-prompt-file` overrides inline prompts for organization runs."""
 
     root = tmp_path / "prompt-file"
     root.mkdir()
@@ -288,10 +300,12 @@ def test_cli_org_prompt_file_overrides_inline_prompt(
         [
             "org",
             str(root),
-            "--prompt",
+            "--classify-prompt",
             "This should be ignored",
-            "--prompt-file",
+            "--classify-prompt-file",
             str(prompt_file),
+            "--structure-prompt",
+            "Structure invoices by accounting period",
         ],
         env=env,
     )
@@ -509,6 +523,31 @@ def test_cli_undo_summary_mode(tmp_path: Path) -> None:
     assert "Rolled back" not in undo_result.output
 
 
+def test_cli_undo_removes_empty_directories(tmp_path: Path) -> None:
+    """Undo should remove directories created during organization when they are empty."""
+
+    root = tmp_path / "undo-cleanup"
+    root.mkdir()
+    original_file = root / "letter.txt"
+    original_file.write_text("hello", encoding="utf-8")
+
+    runner = CliRunner()
+    env = _env_with_home(tmp_path)
+
+    apply_result = runner.invoke(cli, ["org", str(root)], env=env)
+    assert apply_result.exit_code == 0
+
+    destination_dir = root / "documents"
+    destination_file = destination_dir / "letter.txt"
+    assert destination_file.exists()
+
+    undo_result = runner.invoke(cli, ["undo", str(root)], env=env)
+    assert undo_result.exit_code == 0
+
+    assert original_file.exists()
+    assert not destination_dir.exists()
+
+
 def test_cli_status_respects_quiet_default(tmp_path: Path) -> None:
     """Status command should honor the CLI quiet default from configuration."""
 
@@ -529,3 +568,68 @@ def test_cli_status_respects_quiet_default(tmp_path: Path) -> None:
 
     assert status_result.exit_code == 0
     assert status_result.output.strip() == ""
+
+
+def test_cli_org_auto_builds_search_index(tmp_path: Path) -> None:
+    """`dorgy org` should create the Chromadb artifacts by default."""
+
+    root = tmp_path / "search"
+    root.mkdir()
+    (root / "alpha.txt").write_text("alpha", encoding="utf-8")
+
+    runner = CliRunner()
+    env = _env_with_home(tmp_path)
+
+    result = runner.invoke(cli, ["org", str(root)], env=env)
+
+    assert result.exit_code == 0
+    chroma_dir = root / ".dorgy" / "chroma"
+    manifest_path = root / ".dorgy" / "search.json"
+    assert chroma_dir.exists()
+    assert manifest_path.exists()
+    state_data = json.loads((root / ".dorgy" / "state.json").read_text(encoding="utf-8"))
+    assert state_data["search"]["enabled"] is True
+    assert state_data["search"]["last_indexed_at"] is not None
+    manifest = json.loads(manifest_path.read_text(encoding="utf-8"))
+    assert manifest["documents"] >= 1
+
+
+def test_cli_org_with_search_flag_still_builds_index(tmp_path: Path) -> None:
+    """Explicit `--with-search` should continue to build the index."""
+
+    root = tmp_path / "search-flag"
+    root.mkdir()
+    (root / "gamma.txt").write_text("gamma", encoding="utf-8")
+
+    runner = CliRunner()
+    env = _env_with_home(tmp_path)
+
+    result = runner.invoke(cli, ["org", str(root), "--with-search"], env=env)
+
+    assert result.exit_code == 0
+    chroma_dir = root / ".dorgy" / "chroma"
+    manifest_path = root / ".dorgy" / "search.json"
+    assert chroma_dir.exists()
+    assert manifest_path.exists()
+
+
+def test_cli_org_without_search_overrides_auto_enable(tmp_path: Path) -> None:
+    """`--without-search` should suppress indexing even when auto-enable is set."""
+
+    root = tmp_path / "search-disabled"
+    root.mkdir()
+    (root / "beta.txt").write_text("beta", encoding="utf-8")
+
+    runner = CliRunner()
+    env = _env_with_home(tmp_path)
+    config_path = Path(env["HOME"]) / ".dorgy" / "config.yaml"
+    config_path.parent.mkdir(parents=True, exist_ok=True)
+    config_path.write_text("search:\n  auto_enable_org: true\n", encoding="utf-8")
+
+    result = runner.invoke(cli, ["org", str(root), "--without-search"], env=env)
+
+    assert result.exit_code == 0
+    chroma_dir = root / ".dorgy" / "chroma"
+    assert not chroma_dir.exists()
+    state_data = json.loads((root / ".dorgy" / "state.json").read_text(encoding="utf-8"))
+    assert state_data["search"]["enabled"] is False
